package consume

import (
	"encoding/json"
	"fmt"
	"query-service/internal/adapters/brokeradapter"
	"query-service/pkg/errorhandler"
	"query-service/pkg/logger"
)

// HandleMessage gets called when a message is received
func (s *Service) HandleMessage(msg *brokeradapter.Message) {
	// Grab sessionID from the headers
	sessionID, ok := msg.Headers["sessionID"].(string)
	if !ok {
		// TODO: Handle error where there is no session ID supplied
	}

	// Convert the json byte msg to a query string
	query, err := s.queryConverter.ConvertQuery(&msg.Body)
	if err != nil {
		errorhandler.LogError(err, "failed to parse incoming msg to query language") // TODO: send error message to client instead
		return
	}
	// Send the resulting query back to the client
	msgmap := make(map[string]interface{})
	msgmap["type"] = "query_translation_result"
	msgmap["pod"] = podName
	msgmap["values"] = *query
	msgbyte, err := json.Marshal(msgmap)
	if err != nil {
		errorhandler.LogError(err, "Marshalling query_translation_result went wrong!") // TODO: send error message to client instead
		return
	}
	s.producer.PublishMessage(&msgbyte, &sessionID) // TODO: should this be a go routine?

	fmt.Println("Query: " + *query)

	// Make request to database
	// TODO : Generate database seperately
	// execute and retrieve result
	// convert result to general (node-link (?)) format
	result, err := s.requestSender.SendAQLQuery(*query)
	if err != nil {
		logger.Log(err.Error())
		return // TODO: Send message in queue notifying of error
	}

	// Add type indicator to result from database
	resultMsgMap := make(map[string]interface{})
	resultMsgMap["type"] = "query_result"
	resultMsgMap["pod"] = podName
	resultMsgMap["values"] = *result
	resultMsgByte, err := json.Marshal(resultMsgMap)
	if err != nil {
		errorhandler.LogError(err, "Marshalling query_result went wrong!") // TODO: send error message to client instead
		return
	}

	s.producer.PublishMessage(&resultMsgByte, &sessionID)
}
